/**
 * synchro
 * Copyright(c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>
 * MIT Licensed
 */

/**
 * some dependencies
 */
const util = require('util'),
      events = require('events');

/**
 * the step chainer
 */
function Synchro() {
  this._steps = [];
}

// for future use (listen on chains?)
util.inherits(Synchro, events.EventEmitter);

// add a function to be executed
Synchro.prototype.link = function (cb) {
  var step = new Step(cb),
      that = this;
  step.on('next', function () { that.run(); });
  step.on('abort', function () { that.abort(); });
  this._steps.push(step);
  return this;
};

// execute the chain
Synchro.prototype.run = function () {
  if (this._steps.length) {
    this._steps.shift().run();
  } else {
    this.emit('done');
  }
};

// clean up the chain
Synchro.prototype.abort = function () {
  if (this._steps.length) {
    this._steps.forEach(function (obj) {
      delete obj;
    });
    this._steps.length = 0;
  }
  this.emit('aborted');
};

/**
 * functions to be executed are wrapped as steps
 */
function Step(cb) {
  if ('function' != typeof cb) {
    throw new Exception('Invalid callback');
  }
  this._cb = cb;
}

util.inherits(Step, events.EventEmitter);

// execute this function/step
Step.prototype.run = function () {
  // no scope passed, maybe accept one in the future
  this._cb.call(undefined, this);
};

// emit a 'done' event to the chainer
Step.prototype.next = function () {
  this.emit('next');
};

// emit an abort to the chainer
Step.prototype.abort = function () {
  this.emit('abort');
};

exports.create = function () {
  return new Synchro();
};
